@@ -39,7 +39,9 @@ SEFilterStates <- R6::R6Class( # nolint
       checkmate::assert_class(data, "SummarizedExperiment")
       super$initialize(data, data_reactive, dataname, datalabel)
       if (!is.null(datalabel)) {
-        private$dataname_prefixed <- sprintf("%s[['%s']]", dataname, datalabel)
+        private$dataname_prefixed <- sprintf(
+          "%s[['%s']]", private$dataname_prefixed, datalabel
+        )
       }
     },
 
@@ -53,7 +55,7 @@ SEFilterStates <- R6::R6Class( # nolint
     #'
     set_filter_state = function(state) {
       isolate({
-        logger::log_trace("{ class(self)[1] }$set_filter_state initializing, dataname: { private$dataname }")
+        logger::log_debug("{ class(self)[1] }$set_filter_state initializing, dataname: { private$dataname }")
         checkmate::assert_class(state, "teal_slices")
         lapply(state, function(x) {
           checkmate::assert_choice(x$arg, choices = c("subset", "select"), null.ok = TRUE, .var.name = "teal_slice$arg")
@@ -87,7 +89,6 @@ SEFilterStates <- R6::R6Class( # nolint
           }
         )
 
-        logger::log_trace("{ class(self)[1] }$set_filter_state initialized, dataname: { private$dataname }")
         invisible(NULL)
       })
     },
@@ -156,7 +157,7 @@ SEFilterStates <- R6::R6Class( # nolint
       moduleServer(
         id = id,
         function(input, output, session) {
-          logger::log_trace("SEFilterState$srv_add initializing, dataname: { private$dataname }")
+          logger::log_debug("SEFilterState$srv_add initializing, dataname: { private$dataname }")
 
           row_data <- SummarizedExperiment::rowData(data)
           col_data <- SummarizedExperiment::colData(data)
@@ -195,11 +196,11 @@ SEFilterStates <- R6::R6Class( # nolint
             )
           })
 
-          observeEvent(
+          private$session_bindings[[session$ns("avail_row_data_choices")]] <- observeEvent(
             avail_row_data_choices(),
             ignoreNULL = TRUE,
             handlerExpr = {
-              logger::log_trace(paste(
+              logger::log_debug(paste(
                 "SEFilterStates$srv_add@1 updating available row data choices,",
                 "dataname: { private$dataname }"
               ))
@@ -213,18 +214,18 @@ SEFilterStates <- R6::R6Class( # nolint
                 "row_to_add",
                 choices = avail_row_data_choices()
               )
-              logger::log_trace(paste(
+              logger::log_debug(paste(
                 "SEFilterStates$srv_add@1 updated available row data choices,",
                 "dataname: { private$dataname }"
               ))
             }
           )
 
-          observeEvent(
+          private$session_bindings[[session$ns("avail_col_data_choices")]] <- observeEvent(
             avail_col_data_choices(),
             ignoreNULL = TRUE,
             handlerExpr = {
-              logger::log_trace(paste(
+              logger::log_debug(paste(
                 "SEFilterStates$srv_add@2 updating available col data choices,",
                 "dataname: { private$dataname }"
               ))
@@ -238,17 +239,17 @@ SEFilterStates <- R6::R6Class( # nolint
                 "col_to_add",
                 choices = avail_col_data_choices()
               )
-              logger::log_trace(paste(
+              logger::log_debug(paste(
                 "SEFilterStates$srv_add@2 updated available col data choices,",
                 "dataname: { private$dataname }"
               ))
             }
           )
 
-          observeEvent(
+          private$session_bindings[[session$ns("col_to_add")]] <- observeEvent(
             eventExpr = input$col_to_add,
             handlerExpr = {
-              logger::log_trace(
+              logger::log_debug(
                 sprintf(
                   "SEFilterStates$srv_add@3 adding FilterState of column %s to col data, dataname: %s",
                   deparse1(input$col_to_add),
@@ -260,7 +261,7 @@ SEFilterStates <- R6::R6Class( # nolint
                 teal_slice(private$dataname, varname, experiment = private$datalabel, arg = "select")
               ))
 
-              logger::log_trace(
+              logger::log_debug(
                 sprintf(
                   "SEFilterStates$srv_add@3 added FilterState of column %s to col data, dataname: %s",
                   deparse1(varname),
@@ -271,10 +272,10 @@ SEFilterStates <- R6::R6Class( # nolint
           )
 
 
-          observeEvent(
+          private$session_bindings[[session$ns("row_to_add")]] <- observeEvent(
             eventExpr = input$row_to_add,
             handlerExpr = {
-              logger::log_trace(
+              logger::log_debug(
                 sprintf(
                   "SEFilterStates$srv_add@4 adding FilterState of variable %s to row data, dataname: %s",
                   deparse1(input$row_to_add),
@@ -286,7 +287,7 @@ SEFilterStates <- R6::R6Class( # nolint
                 teal_slice(private$dataname, varname, experiment = private$datalabel, arg = "subset")
               ))
 
-              logger::log_trace(
+              logger::log_debug(
                 sprintf(
                   "SEFilterStates$srv_add@4 added FilterState of variable %s to row data, dataname: %s",
                   deparse1(varname),
@@ -296,7 +297,13 @@ SEFilterStates <- R6::R6Class( # nolint
             }
           )
 
-          logger::log_trace("SEFilterState$srv_add initialized, dataname: { private$dataname }")
+          # Extra observer that clears all input values in session
+          private$session_bindings[[session$ns("inputs")]] <- list(
+            destroy = function() {
+              lapply(session$ns(names(input)), .subset2(input, "impl")$.values$remove)
+            }
+          )
+
           NULL
         }
       )
